% ---------------------------------------------------------------------------- %
% ---------------------------------------------------------------------------- %
%
% Module: rwab1__human
% Author: Ralph Becket <rwab1@cam.sri.com>
%
% Basically, this does exactly what it says on the tin.  The input routines
% are somewhat crude and unforgiving, but that's a job for a separate
% library.
%
% ---------------------------------------------------------------------------- %
% ---------------------------------------------------------------------------- %



:- module rwab1__human.

:- interface.

:- import_module io.

:- type human.

:- instance player(human) where [
		pred(place_ship/8)		is human__place_ship,
		pred(guess/5)			is human__guess,
		pred(your_shot_result/5)	is human__your_shot_result,
		pred(their_shot_result/5)	is human__their_shot_result
	].

	% Returns the initial state for use by the human player.
:- pred human__human(human::out) is det.

:- pred human__place_ship(locn, ship, locn, dirn, human, human, io__state, io__state).
:- mode human__place_ship(in, in, out, out, in, out, di, uo) is det.

:- pred human__guess(locn, human, human, io__state, io__state).
:- mode human__guess(out, in, out, di, uo) is det.

:- pred human__your_shot_result(battle__result, human, human, io__state, io__state).
:- mode human__your_shot_result(in, in, out, di, uo) is det.

:- pred human__their_shot_result(battle__result, human, human, io__state, io__state).
:- mode human__their_shot_result(in, in, out, di, uo) is det.

% ---------------------------------------------------------------------------- %
% ---------------------------------------------------------------------------- %

:- implementation.

:- import_module list, string, int, require.
:- import_module rwab1__view.

:- type human
	--->	human(view).

% ---------------------------------------------------------------------------- %

human__human(human(View)) :-
	view__init(View).

% ---------------------------------------------------------------------------- %

human__place_ship(BoardSize, Ship, Locn, Dirn, human(View0), human(View)) -->

	io__write_many([
		s("\nYou need to place a "),
		s(ship_name(Ship)),
		s(" on the board.\n")
	]),

	get_locn(BoardSize, Locn0),
	get_dirn(Dirn0),

	{ view__place_local_ship(BoardSize,Ship,Locn0,Dirn0,Result,View0,View1) },
	(
		{ Result = error(Msg) },

		io__print(Msg),
		io__nl,

		human__place_ship(BoardSize, Ship, Locn, Dirn, human(View1), human(View))
	;
		{ Result = ok },

		{ Locn = Locn0 },
		{ Dirn = Dirn0 },
		{ View = View1 },

		view__display(View)
	).

% ---------------------------------------------------------------------------- %

:- pred get_locn(locn, locn, io__state, io__state).
:- mode get_locn(in, out, di, uo) is det.

get_locn(locn(Cols, Rows), locn(X, Y)) -->
	get_coord([s("- Row (0.."), i(Rows-1), s("): ")], 0, Rows, Y),
	get_coord([s("- Col (0.."), i(Cols-1), s("): ")], 0, Cols, X).

% ---------------------------------------------------------------------------- %

:- pred get_coord(list(io__poly_type), int, int, int, io__state, io__state).
:- mode get_coord(in, in, in, out, di, uo) is det.

	% XXX Coding this predicate showed me that there is need for
	% a decent non-Mercury parser module.  This is pernickety code.

get_coord(Prompt, Lo, Hi, N) -->
	io__write_many(Prompt),
	io__read_line_as_string(IO_S),

	(
		{ IO_S = ok(S0) },
		{ string__remove_suffix(S0, "\n", S) },
		{ string__to_int(S, N0)} , 
		{ Lo =< N0, N0 < Hi }
	->
		{ N = N0 }
	;
		{ IO_S = ok(_) }
	->
		io__print("* That's not a valid response.\n"),
		get_coord(Prompt, Lo, Hi, N)
	;
		{ error("* rwab1__human__get_coord/6: it's all gone pear shaped.") }
	).

% ---------------------------------------------------------------------------- %

:- pred get_dirn(dirn, io__state, io__state).
:- mode get_dirn(out, di, uo) is det.

get_dirn(Dirn) -->
	io__print("- Across or down [ad]: "),
	io__read_line_as_string(IO_S),

	( { IO_S = ok("a\n") } ->
		{ Dirn = across }

	; { IO_S = ok("d\n") } ->
		{ Dirn = down }

	; { IO_S = ok(_) } ->
		io__print("* just an `a' or a `d', please.\n"),
		get_dirn(Dirn)

	;
		{ error("* rwab1__human__get_dirn/3: time to bail out, Ginger!") }
	).

% ---------------------------------------------------------------------------- %

human__guess(Locn, human(View), human(View)) -->
	{ view__board_size(View, BoardSize) },
	get_locn(BoardSize, Locn).

% ---------------------------------------------------------------------------- %

human__your_shot_result(Result, human(View0), human(View)) -->
	(
		{ Result = fleet_destroyed },
		io__print("\n*** YOU WON! ***\n"),
		{ View = View0 }
	;
		{ Result = miss(Locn) },
		io__print("\n*splash*  You missed.  Waster.\n"),
		{ update_enemy_view(Locn, miss, View0, View) },
		view__display(View)
	;
		{ Result = sink(Locn, Ship) },
		io__print("\n*Glug... glug...*  You sank an enemy "),
		io__print(ship_name(Ship)),
		io__print("!\n"),
		{ update_enemy_view(Locn, hit, View0, View) },
		view__display(View)
	;
		{ Result = hit(Locn) },
		io__print("\n*BOOM*  We will crush the foe yet!\n"),
		{ update_enemy_view(Locn, hit, View0, View) },
		view__display(View)
	).

% ---------------------------------------------------------------------------- %

human__their_shot_result(Result, human(View0), human(View)) -->
	(
		{ Result = fleet_destroyed },
		io__print("\n*** WE MUST FLEE - THE ENEMY HAS WON! ***\n"),
		{ View = View0 }
	;
		{ Result = miss(Locn) },
		io__print("\n*splash*  Feeble shooting from the enemy.\n"),
		{ update_local_view(Locn, miss, View0, View) },
		view__display(View)
	;
		{ Result = sink(Locn, Ship) },
		io__print("\n*Glug... glug...*  The enemy sank your "),
		io__print(ship_name(Ship)),
		io__print("!\n"),
		{ update_local_view(Locn, hit, View0, View) },
		view__display(View)
	;
		{ Result = hit(Locn) },
		io__print("\n*BOOM*  The enemy strikes a mighty blow.\n"),
		{ update_local_view(Locn, hit, View0, View) },
		view__display(View)
	).

% ---------------------------------------------------------------------------- %
% Utility predicate.

:- func ship_name(battle__ship) = string.
:- mode ship_name(in) = out is det.

ship_name(battle__battleship) = "battleship".
ship_name(battle__cruiser   ) = "cruiser".
ship_name(battle__destroyer ) = "destroyer".

