%------------------------------------------------------------------------------%
%------------------------------------------------------------------------------%
%
% Module: silly
% Author: Peter Ross (petdr@cs.mu.oz.au)
%
% A silly player for the game battleships.  This player just guesses the
% last guess made by the opposing player.
%
% This source file is hereby placed in the public domain.  -petdr.
% Necessary so that this brilliant player can go in the samples
% directory.
%
%------------------------------------------------------------------------------%
%------------------------------------------------------------------------------%

:- module silly.

:- interface.

:- import_module battle.

:- type silly.

:- instance player(silly) where [
		pred(place_ship/8) is silly__place_ship,
		pred(guess/5) is silly__guess,
		pred(your_shot_result/5) is silly__your_shot_result,
		pred(their_shot_result/5) is silly__their_shot_result
	].

	% Returns the initial state for use by the silly player.
:- pred silly(silly::out) is det.

%------------------------------------------------------------------------------%

:- implementation.

:- import_module int, io.

:- type silly
	--->	silly(
			locn,		% Opponents guess.
			int		% Which x coord to put my next ship
		).

%------------------------------------------------------------------------------%

	% Initialise my first guess to be (0,0) and put my first ship at
	% x coordinate 0.
silly(silly(locn(0,0), 0)).

%------------------------------------------------------------------------------%

	% Place my ships across the top.  Who would think to look there.
:- pred place_ship(locn, ship, locn, dirn, silly, silly, io__state, io__state).
:- mode place_ship(in, in, out, out, in, out, di, uo) is det.

place_ship(_BoardSize, _Ship, Locn, Dirn, State0, State) -->
	{ State0 = silly(OpponentsGuess, X) },
	{ Locn = locn(X, 0) },
	{ Dirn = down },
	{ State = silly(OpponentsGuess, X+1) }.

%------------------------------------------------------------------------------%

	% Guess the whatever my opponents last guess was.
:- pred guess(locn::out, silly::in, silly::out,
		io__state::di, io__state::uo) is det.

guess(OpponentsGuess, silly(OpponentsGuess, Col), silly(OpponentsGuess, Col))-->
	[].

%------------------------------------------------------------------------------%

	% Don't care what happened to my shot.
:- pred your_shot_result(battle__result::in, silly::in, silly::out,
		io__state::di, io__state::uo) is det.

your_shot_result(_Result, State, State) --> [].

%------------------------------------------------------------------------------%

	% Record the opponents last guess.
:- pred their_shot_result(battle__result::in, silly::in, silly::out,
		io__state::di, io__state::uo) is det.

their_shot_result(hit(Locn), silly(_, Col), silly(Locn, Col)) --> [].
their_shot_result(sink(Locn, _), silly(_, Col), silly(Locn, Col)) --> [].
their_shot_result(miss(Locn), silly(_, Col), silly(Locn, Col)) --> [].
their_shot_result(fleet_destroyed, silly(Locn, Col), silly(Locn, Col)) --> [].

%------------------------------------------------------------------------------%
%------------------------------------------------------------------------------%
